@@ -2,11 +2,16 @@ package dnszone
 
 import (
 	"context"
+	"fmt"
 	"testing"
 
+	"github.com/aws/aws-sdk-go/aws/awserr"
+	"github.com/golang/mock/gomock"
 	log "github.com/sirupsen/logrus"
 	"github.com/stretchr/testify/assert"
+	"github.com/stretchr/testify/require"
 
+	corev1 "k8s.io/api/core/v1"
 	"k8s.io/apimachinery/pkg/types"
 	"k8s.io/client-go/kubernetes/scheme"
 
@@ -16,6 +21,8 @@ import (
 	azuremock "github.com/openshift/hive/pkg/azureclient/mock"
 	controllerutils "github.com/openshift/hive/pkg/controller/utils"
 	gcpmock "github.com/openshift/hive/pkg/gcpclient/mock"
+	testdnszone "github.com/openshift/hive/pkg/test/dnszone"
+	testgeneric "github.com/openshift/hive/pkg/test/generic"
 )
 
 // TestReconcileDNSProviderForAWS tests that ReconcileDNSProvider reacts properly under different reconciliation states on AWS.
@@ -127,6 +134,23 @@ func TestReconcileDNSProviderForAWS(t *testing.T) {
 				assert.False(t, controllerutils.HasFinalizer(zone, hivev1.FinalizerDNSZone))
 			},
 		},
+		{
+			name: "Delete DNSZone without status",
+			dnsZone: func() *hivev1.DNSZone {
+				dz := validDNSZoneBeingDeleted()
+				dz.Status.AWS = nil
+				return dz
+			}(),
+			setupAWSMock: func(expect *mock.MockClientMockRecorder) {
+				mockGetResourcePages(expect)
+				mockAWSZoneExists(expect, validDNSZoneWithAdditionalTags())
+				mockExistingAWSTags(expect)
+				mockDeleteAWSZone(expect)
+			},
+			validateZone: func(t *testing.T, zone *hivev1.DNSZone) {
+				assert.False(t, controllerutils.HasFinalizer(zone, hivev1.FinalizerDNSZone))
+			},
+		},
 		{
 			name:            "Existing zone, link to parent, reachable SOA",
 			dnsZone:         validDNSZoneWithLinkToParent(),
@@ -248,8 +272,17 @@ func TestReconcileDNSProviderForGCP(t *testing.T) {
 			},
 		},
 		{
-			name:    "Delete managed zone",
-			dnsZone: validDNSZoneBeingDeleted(),
+			name: "Delete managed zone",
+			dnsZone: testdnszone.BasicBuilder().
+				Options(
+					testdnszone.WithGCPPlatform("testDNSZone"),
+				).
+				GenericOptions(
+					testgeneric.WithNamespace("testNamespace"),
+					testgeneric.WithName("testDNSZone"),
+					testgeneric.Deleted(),
+				).
+				Build(),
 			setupGCPMock: func(expect *gcpmock.MockClientMockRecorder) {
 				mockGCPZoneExists(expect)
 				mockDeleteGCPZone(expect)
@@ -307,14 +340,15 @@ func TestReconcileDNSProviderForGCP(t *testing.T) {
 			// This is necessary for the mocks to report failures like methods not being called an expected number of times.
 			defer mocks.mockCtrl.Finish()
 
-			setFakeDNSZoneInKube(mocks, tc.dnsZone)
+			err := setFakeDNSZoneInKube(mocks, tc.dnsZone)
+			require.NoError(t, err, "failed to create DNSZone into fake client")
 
 			if tc.setupGCPMock != nil {
 				tc.setupGCPMock(mocks.mockGCPClient.EXPECT())
 			}
 
 			// Act
-			_, err := r.reconcileDNSProvider(zr, tc.dnsZone)
+			_, err = r.reconcileDNSProvider(zr, tc.dnsZone)
 
 			// Assert
 			if tc.errorExpected {
@@ -344,7 +378,7 @@ func TestReconcileDNSProviderForAzure(t *testing.T) {
 	cases := []struct {
 		name            string
 		dnsZone         *hivev1.DNSZone
-		setupAzureMock  func(*azuremock.MockClientMockRecorder)
+		setupAzureMock  func(*gomock.Controller, *azuremock.MockClientMockRecorder)
 		validateZone    func(*testing.T, *hivev1.DNSZone)
 		errorExpected   bool
 		soaLookupResult bool
@@ -352,7 +386,7 @@ func TestReconcileDNSProviderForAzure(t *testing.T) {
 		{
 			name:    "DNSZone without finalizer",
 			dnsZone: validAzureDNSZoneWithoutFinalizer(),
-			setupAzureMock: func(expect *azuremock.MockClientMockRecorder) {
+			setupAzureMock: func(_ *gomock.Controller, expect *azuremock.MockClientMockRecorder) {
 				mockAzureZoneExists(expect)
 			},
 			validateZone: func(t *testing.T, zone *hivev1.DNSZone) {
@@ -362,7 +396,7 @@ func TestReconcileDNSProviderForAzure(t *testing.T) {
 		{
 			name:    "Create Managed Zone",
 			dnsZone: validAzureDNSZone(),
-			setupAzureMock: func(expect *azuremock.MockClientMockRecorder) {
+			setupAzureMock: func(_ *gomock.Controller, expect *azuremock.MockClientMockRecorder) {
 				mockAzureZoneDoesntExist(expect)
 				mockCreateAzureZone(expect)
 			},
@@ -373,7 +407,7 @@ func TestReconcileDNSProviderForAzure(t *testing.T) {
 		{
 			name:    "Adopt existing zone",
 			dnsZone: validAzureDNSZone(),
-			setupAzureMock: func(expect *azuremock.MockClientMockRecorder) {
+			setupAzureMock: func(_ *gomock.Controller, expect *azuremock.MockClientMockRecorder) {
 				mockAzureZoneExists(expect)
 			},
 			validateZone: func(t *testing.T, zone *hivev1.DNSZone) {
@@ -383,9 +417,9 @@ func TestReconcileDNSProviderForAzure(t *testing.T) {
 		{
 			name:    "Delete managed zone",
 			dnsZone: validAzureDNSZoneBeingDeleted(),
-			setupAzureMock: func(expect *azuremock.MockClientMockRecorder) {
+			setupAzureMock: func(mockCtrl *gomock.Controller, expect *azuremock.MockClientMockRecorder) {
 				mockAzureZoneExists(expect)
-				mockDeleteAzureZone(expect)
+				mockDeleteAzureZone(mockCtrl, expect)
 			},
 			validateZone: func(t *testing.T, zone *hivev1.DNSZone) {
 				assert.False(t, controllerutils.HasFinalizer(zone, hivev1.FinalizerDNSZone))
@@ -394,7 +428,7 @@ func TestReconcileDNSProviderForAzure(t *testing.T) {
 		{
 			name:    "Delete non-existent managed zone",
 			dnsZone: validAzureDNSZoneBeingDeleted(),
-			setupAzureMock: func(expect *azuremock.MockClientMockRecorder) {
+			setupAzureMock: func(_ *gomock.Controller, expect *azuremock.MockClientMockRecorder) {
 				mockAzureZoneDoesntExist(expect)
 			},
 			validateZone: func(t *testing.T, zone *hivev1.DNSZone) {
@@ -405,7 +439,7 @@ func TestReconcileDNSProviderForAzure(t *testing.T) {
 			name:            "Existing zone, link to parent, reachable SOA",
 			dnsZone:         validAzureDNSZoneWithLinkToParent(),
 			soaLookupResult: true,
-			setupAzureMock: func(expect *azuremock.MockClientMockRecorder) {
+			setupAzureMock: func(_ *gomock.Controller, expect *azuremock.MockClientMockRecorder) {
 				mockAzureZoneExists(expect)
 			},
 			validateZone: func(t *testing.T, zone *hivev1.DNSZone) {
@@ -443,7 +477,7 @@ func TestReconcileDNSProviderForAzure(t *testing.T) {
 			setFakeDNSZoneInKube(mocks, tc.dnsZone)
 
 			if tc.setupAzureMock != nil {
-				tc.setupAzureMock(mocks.mockAzureClient.EXPECT())
+				tc.setupAzureMock(mocks.mockCtrl, mocks.mockAzureClient.EXPECT())
 			}
 
 			// Act
@@ -468,3 +502,104 @@ func TestReconcileDNSProviderForAzure(t *testing.T) {
 		})
 	}
 }
+
+func TestSetConditionsForErrorForAWS(t *testing.T) {
+
+	log.SetLevel(log.DebugLevel)
+
+	cases := []struct {
+		name            string
+		dnsZone         *hivev1.DNSZone
+		error           error
+		expectCondition *hivev1.DNSZoneCondition
+	}{
+		{
+			name:    "Set InsufficientCredentialsCondition on DNSZone for AccessDeniedException error",
+			dnsZone: validDNSZone(),
+			error:   testAccessDeniedExceptionError(),
+			expectCondition: &hivev1.DNSZoneCondition{
+				Type:    hivev1.InsufficientCredentialsCondition,
+				Status:  corev1.ConditionTrue,
+				Reason:  accessDeniedReason,
+				Message: "User: arn:aws:iam::0123456789:user/testAdmin is not authorized to perform: tag:GetResources with an explicit deny",
+			},
+		},
+		{
+			name:    "Set AuthenticationFailureCondition on DNSZone for UnrecognizedClientException error",
+			dnsZone: validDNSZone(),
+			error:   testUnrecognizedClientExceptionError(),
+			expectCondition: &hivev1.DNSZoneCondition{
+				Type:    hivev1.AuthenticationFailureCondition,
+				Status:  corev1.ConditionTrue,
+				Reason:  authenticationFailedReason,
+				Message: "The security token included in the request is invalid.",
+			},
+		},
+		{
+			name:    "Set AuthenticationFailureCondition on DNSZone for InvalidSignatureException error",
+			dnsZone: validDNSZone(),
+			error:   testInvalidSignatureExceptionError(),
+			expectCondition: &hivev1.DNSZoneCondition{
+				Type:    hivev1.AuthenticationFailureCondition,
+				Status:  corev1.ConditionTrue,
+				Reason:  authenticationFailedReason,
+				Message: "The request signature we calculated does not match the signature you provided. Check your AWS Secret Access Key and signing method. Consult the service documentation for details.",
+			},
+		},
+	}
+	for _, tc := range cases {
+		t.Run(tc.name, func(t *testing.T) {
+			// Arrange
+			mocks := setupDefaultMocks(t)
+
+			zr, _ := NewAWSActuator(
+				log.WithField("controller", ControllerName),
+				validAWSSecret(),
+				tc.dnsZone,
+				fakeAWSClientBuilder(mocks.mockAWSClient),
+			)
+
+			zr.dnsZone = tc.dnsZone
+
+			// This is necessary for the mocks to report failures like methods not being called an expected number of times.
+			defer mocks.mockCtrl.Finish()
+
+			// Act
+			zr.SetConditionsForError(tc.error)
+
+			// Assert
+			if tc.expectCondition != nil {
+				for _, cond := range zr.dnsZone.Status.Conditions {
+					assert.Equal(t, cond.Type, tc.expectCondition.Type)
+					assert.Equal(t, cond.Status, tc.expectCondition.Status)
+					assert.Equal(t, cond.Reason, tc.expectCondition.Reason)
+					assert.Equal(t, cond.Message, tc.expectCondition.Message)
+				}
+			} else {
+				// Assuming if you didn't expect a condition, there shouldn't be any.
+				assert.Equal(t, 0, len(zr.dnsZone.Status.Conditions))
+			}
+		})
+	}
+}
+
+func testAccessDeniedExceptionError() error {
+	accessDeniedErr := awserr.New("AccessDeniedException",
+		"User: arn:aws:iam::0123456789:user/testAdmin is not authorized to perform: tag:GetResources with an explicit deny",
+		fmt.Errorf("User: arn:aws:iam::0123456789:user/testAdmin is not authorized to perform: tag:GetResources with an explicit deny"))
+	return accessDeniedErr
+}
+
+func testUnrecognizedClientExceptionError() error {
+	unrecognizedClientErr := awserr.New("UnrecognizedClientException",
+		"The security token included in the request is invalid.",
+		fmt.Errorf("The security token included in the request is invalid"))
+	return unrecognizedClientErr
+}
+
+func testInvalidSignatureExceptionError() error {
+	invalidSignatureErr := awserr.New("InvalidSignatureException",
+		"The request signature we calculated does not match the signature you provided. Check your AWS Secret Access Key and signing method. Consult the service documentation for details.",
+		fmt.Errorf("The request signature we calculated does not match the signature you provided. Check your AWS Secret Access Key and signing method. Consult the service documentation for details"))
+	return invalidSignatureErr
+}
