@@ -2,6 +2,7 @@ package install
 
 import (
 	"fmt"
+	"strings"
 
 	"github.com/pkg/errors"
 	log "github.com/sirupsen/logrus"
@@ -16,6 +17,7 @@ import (
 	hivev1 "github.com/openshift/hive/pkg/apis/hive/v1"
 	"github.com/openshift/hive/pkg/constants"
 	"github.com/openshift/hive/pkg/controller/images"
+	"github.com/openshift/hive/pkg/controller/utils"
 )
 
 const (
@@ -24,6 +26,7 @@ const (
 	gcpAuthDir         = "/.gcp"
 	gcpAuthFile        = gcpAuthDir + "/" + constants.GCPCredentialsName
 	openStackCloudsDir = "/etc/openstack"
+	openStackCADir     = "/etc/openstack-ca"
 	vsphereCloudsDir   = "/vsphere"
 	ovirtCloudsDir     = "/.ovirt"
 	ovirtCADir         = "/.ovirt-ca"
@@ -49,8 +52,7 @@ func InstallerPodSpec(
 	provisionName string,
 	releaseImage string,
 	serviceAccountName string,
-	pvcName string,
-	skipGatherLogs bool,
+	extraEnvVars []corev1.EnvVar,
 ) (*corev1.PodSpec, error) {
 
 	if cd.Spec.Provisioning == nil {
@@ -72,6 +74,9 @@ func InstallerPodSpec(
 			Value: "hive",
 		},
 	}
+
+	env = append(env, extraEnvVars...)
+
 	volumes := []corev1.Volume{
 		{
 			Name: "output",
@@ -79,6 +84,12 @@ func InstallerPodSpec(
 				EmptyDir: &corev1.EmptyDirVolumeSource{},
 			},
 		},
+		{
+			Name: "logs",
+			VolumeSource: corev1.VolumeSource{
+				EmptyDir: &corev1.EmptyDirVolumeSource{},
+			},
+		},
 		{
 			Name: "installconfig",
 			VolumeSource: corev1.VolumeSource{
@@ -101,6 +112,10 @@ func InstallerPodSpec(
 			Name:      "output",
 			MountPath: "/output",
 		},
+		{
+			Name:      "logs",
+			MountPath: "/logs",
+		},
 		{
 			Name:      "installconfig",
 			MountPath: "/installconfig",
@@ -181,6 +196,20 @@ func InstallerPodSpec(
 			Name:      "openstack",
 			MountPath: openStackCloudsDir,
 		})
+		if cd.Spec.Platform.OpenStack.CertificatesSecretRef != nil && cd.Spec.Platform.OpenStack.CertificatesSecretRef.Name != "" {
+			volumes = append(volumes, corev1.Volume{
+				Name: "openstack-certificates",
+				VolumeSource: corev1.VolumeSource{
+					Secret: &corev1.SecretVolumeSource{
+						SecretName: cd.Spec.Platform.OpenStack.CertificatesSecretRef.Name,
+					},
+				},
+			})
+			volumeMounts = append(volumeMounts, corev1.VolumeMount{
+				Name:      "openstack-certificates",
+				MountPath: openStackCADir,
+			})
+		}
 	case cd.Spec.Platform.VSphere != nil:
 		volumes = append(volumes, corev1.Volume{
 			Name: "vsphere-certificates",
@@ -260,28 +289,6 @@ func InstallerPodSpec(
 		)
 	}
 
-	if !skipGatherLogs {
-		// Add a volume where we will store full logs from both the installer, and the
-		// cluster itself (assuming we made it far enough).
-		volumes = append(volumes, corev1.Volume{
-			Name: "logs",
-			VolumeSource: corev1.VolumeSource{
-				PersistentVolumeClaim: &corev1.PersistentVolumeClaimVolumeSource{
-					ClaimName: pvcName,
-				},
-			},
-		})
-		volumeMounts = append(volumeMounts, corev1.VolumeMount{
-			Name:      "logs",
-			MountPath: "/logs",
-		})
-	} else {
-		env = append(env, corev1.EnvVar{
-			Name:  constants.SkipGatherLogsEnvVar,
-			Value: "true",
-		})
-	}
-
 	if cd.Spec.Provisioning.SSHPrivateKeySecretRef != nil {
 		volumes = append(volumes, corev1.Volume{
 			Name: "sshkeys",
@@ -320,6 +327,14 @@ func InstallerPodSpec(
 		})
 	}
 
+	// Signal to fake an installation:
+	if utils.IsFakeCluster(cd) {
+		env = append(env, corev1.EnvVar{
+			Name:  constants.FakeClusterInstallEnvVar,
+			Value: "true",
+		})
+	}
+
 	if cd.Status.InstallerImage == nil {
 		return nil, fmt.Errorf("installer image not resolved")
 	}
@@ -339,6 +354,10 @@ func InstallerPodSpec(
 		// Add oVirt certificates to CA trust.
 		hiveArg = fmt.Sprintf("cp -vr %s/. /etc/pki/ca-trust/source/anchors/ && update-ca-trust && %s", ovirtCADir, hiveArg)
 	}
+	if cd.Spec.Platform.OpenStack != nil && cd.Spec.Platform.OpenStack.CertificatesSecretRef != nil && cd.Spec.Platform.OpenStack.CertificatesSecretRef.Name != "" {
+		// Add OpenStack certificates to CA trust.
+		hiveArg = fmt.Sprintf("cp -vr %s/. /etc/pki/ca-trust/source/anchors/ && update-ca-trust && %s", openStackCADir, hiveArg)
+	}
 
 	// This is used when scheduling the installer pod. It ensures that installer pods don't overwhelm
 	// a given node's memory.
@@ -582,10 +601,6 @@ func completeAzureDeprovisionJob(req *hivev1.ClusterDeprovision, job *batchv1.Jo
 			VolumeMounts: volumeMounts,
 		},
 	}
-	if req.Spec.Platform.Azure.ResourceGroupName != "" {
-		containers[0].Args = append(containers[0].Args, "--resource-group", req.Spec.Platform.Azure.ResourceGroupName)
-	}
-
 	job.Spec.Template.Spec.Containers = containers
 	job.Spec.Template.Spec.Volumes = volumes
 
@@ -652,25 +667,45 @@ func completeOpenStackDeprovisionJob(req *hivev1.ClusterDeprovision, job *batchv
 		Name:      "openstack",
 		MountPath: openStackCloudsDir,
 	})
+	if req.Spec.Platform.OpenStack.CertificatesSecretRef != nil && req.Spec.Platform.OpenStack.CertificatesSecretRef.Name != "" {
+		volumes = append(volumes, corev1.Volume{
+			Name: "openstack-certificates",
+			VolumeSource: corev1.VolumeSource{
+				Secret: &corev1.SecretVolumeSource{
+					SecretName: req.Spec.Platform.OpenStack.CertificatesSecretRef.Name,
+				},
+			},
+		})
+		volumeMounts = append(volumeMounts, corev1.VolumeMount{
+			Name:      "openstack-certificates",
+			MountPath: openStackCADir,
+		})
+	}
+	cmd := []string{"/usr/bin/hiveutil"}
+	args := []string{
+		"deprovision",
+		"openstack",
+		"--loglevel",
+		"debug",
+		"--creds-dir",
+		openStackCloudsDir,
+		"--cloud",
+		req.Spec.Platform.OpenStack.Cloud,
+		req.Spec.InfraID,
+	}
+	if req.Spec.Platform.OpenStack.CertificatesSecretRef != nil && req.Spec.Platform.OpenStack.CertificatesSecretRef.Name != "" {
+		args = []string{fmt.Sprintf("cp -vr %s/. /etc/pki/ca-trust/source/anchors/ && update-ca-trust && %s", openStackCADir, strings.Join(append(cmd, args...), " "))}
+		cmd = []string{"/bin/sh", "-c"}
+	}
 	containers := []corev1.Container{
 		{
 			Name:            "deprovision",
 			Image:           images.GetHiveImage(),
 			ImagePullPolicy: images.GetHiveImagePullPolicy(),
 			Env:             env,
-			Command:         []string{"/usr/bin/hiveutil"},
-			Args: []string{
-				"deprovision",
-				"openstack",
-				"--loglevel",
-				"debug",
-				"--creds-dir",
-				openStackCloudsDir,
-				"--cloud",
-				req.Spec.Platform.OpenStack.Cloud,
-				req.Spec.InfraID,
-			},
-			VolumeMounts: volumeMounts,
+			Command:         cmd,
+			Args:            args,
+			VolumeMounts:    volumeMounts,
 		},
 	}
 	job.Spec.Template.Spec.Containers = containers
